"""Contains constants used in the 'config.ini' file.

Much of the logic that uses these constants relies very much on the
precise values these constants take. If you want to change these values,
you should look at every occurance of their use to ensure that things
won't break.
"""

NO = 0
YES = 1
PROMPT = 2
